/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.manager.SystemConsoleManagerTest.java
 * Created on: Nov 19, 2016
 */
package fr.bigeon.gclc.manager;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.IOException;
import java.io.InputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.nio.charset.Charset;

import org.junit.Test;

/** <p>
 * TODO
 *
 * @author Emmanuel Bigeon */
public class SystemConsoleManagerTest {

    /** Test method for
     * {@link fr.bigeon.gclc.manager.SystemConsoleManager#prompt()}. */
    @Test
    public final void testPrompt() {

        try {
            PipedOutputStream outStream = new PipedOutputStream();
            InputStream in = new PipedInputStream(outStream);
            final PrintStream out = new PrintStream(outStream);
            final String test = "test";
            SystemConsoleManager manager = new SystemConsoleManager(System.out,
                    in, Charset.forName("UTF-8"));

            Thread th = new Thread(new Runnable() {

                @SuppressWarnings("synthetic-access")
                @Override
                public void run() {
                    out.println(test);
                }
            });

            th.start();
            assertEquals(test, manager.prompt());

            th.join();
        } catch (IOException | InterruptedException e) {
            assertNull(e);
        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.manager.SystemConsoleManager#setPrompt(java.lang.String)}. */
    @Test
    public final void testSetPrompt() {
        SystemConsoleManager manager = new SystemConsoleManager();
        String prt = "++";
        manager.setPrompt(prt);
        assertEquals(prt, manager.getPrompt());
    }

    /** Test method for
     * {@link fr.bigeon.gclc.manager.SystemConsoleManager#isClosed()}. */
    @Test
    public final void testIsClosed() {
        try {
            PipedOutputStream outStream = new PipedOutputStream();
            InputStream in = new PipedInputStream(outStream);
            final PrintStream out = new PrintStream(outStream);
            final String test = "test";
            SystemConsoleManager manager = new SystemConsoleManager(System.out,
                    in, Charset.forName("UTF-8"));

            Thread th = new Thread(new Runnable() {

                @SuppressWarnings("synthetic-access")
                @Override
                public void run() {
                    out.println(test);
                }
            });

            th.start();
            assertEquals(test, manager.prompt());
            assertFalse(manager.isClosed());
            manager.close();
            assertTrue(manager.isClosed());
            try {
                manager.prompt();
                fail("prompt on closed manager");
            } catch (IOException e) {
                assertNotNull(e);
            }
            try {
                manager.println(test);
                fail("prompt on closed manager");
            } catch (IOException e) {
                assertNotNull(e);
            }
            th.join();
        } catch (IOException | InterruptedException e) {
            assertNull(e);
        }
    }

}
